﻿# 4_update_folders.ps1 - Reads the file created in 3_prepare_folders.ps1.
#                        Renames all folders found in the file.
#
# Version 1.1: check for write-protected archive stores added
#              pkv3 value not passed to CmdClient anymore when value is empty
# Version 1.0: initial version
#
# Copyright (c) 2017 - 2022 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# https://help.mailstore.com/en/server/PowerShell_API_Wrapper_Tutorial

# API Wrapper in same directory is used, nothing to configure here
Import-Module (Join-Path $PSScriptRoot "MS.PS.Lib.psd1") -ErrorAction Stop

######################################################
# Adjust these values according to your installation #
######################################################

# Username of an MailStore SPE Admin, default is "admin"
$speUsername = "admin"

# If password is empty, an interactive prompt will ask for it
$spePassword = ''

# The hostname of the MailStore Management Server, default is "localhost"
$mgmtHostname = "localhost"

# TCP Port of the Management API, API must be enabled in the MailStore SPE Configuration, default is 8474
$mgmtPort = 8474

# Set this to $true when untrusted certificates are used, default is $false
$mgmtAcceptAllCertificates = $false

# Install the "E-mail Archive Client" and adjust the path to the MailStoreCmd.exe
$MailStoreCmd = "C:\Program Files (x86)\E-mail Archive Client\MailStoreCmd.exe"

# Username of an MailStore SPE Instance administrator, default is "admin", must not the global SPE admin!
$instanceUsername = "admin"

# Password of the instance administrator, if password is empty, an interactive prompt will ask for it
$instancePassword = ''

# The hostname of the MailStore SPE Client Access Server (CAS), default is "localhost"
$casHostname = "localhost"

# TCP Port of the Client Access, default is 443
$casPort = 443

# When using a certificate on the CAS that is not trusted, the certificate thumbprint must be entered here
$casHttpThumbprint = ''

# Instance ID of the instance to work on
$instanceID = "instanceID"

# File where the folder list is stored
$filename = Join-Path $env:HOMEDRIVE (Join-Path $env:HOMEPATH ("SPE-" + $instanceID + "-folders.txt"))

################### DO NOT CHANGE ANYTHING BELOW THIS LINE ###################

function Has-Succeeded($result) {
    if ($result.statusCode -ne "succeeded") {
        Write-Host ("Status  " + $result.statusCode)
        Write-Host ("Message " + $result.error.message)
        Write-Host ("Details " + $result.error.details)
        return $false
    }
    return $true
}

function My-Exit() {
    if (-not (Test-Path variable:global:psISE)) {
        Read-Host -Prompt "Press [Enter] to exit script"
    }
    exit
}

if (-not (Test-Path $MailStoreCmd -PathType Leaf)){
    Write-Host ($MailStoreCmd + " does not exist.")
    My-Exit
}

if (-not (Test-Path $filename -PathType Leaf)){
    Write-Host ($filename + " does not exist.")
    My-Exit
}

if ($spePassword -ceq ''){
    $spePassword = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR((Read-Host -AsSecureString -Prompt "Enter MailStore SPE Password for $speUsername")))
}

if ($instancePassword -ceq ''){
    $instancePassword = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR((Read-Host -AsSecureString -Prompt "Enter MailStore Instance Password for $instanceUsername ($instanceID)")))
}

# CmdClient ("command", "parameters")
function CmdClient([array]$command) {
    if (-not $command) {
        Write-Host "Error: No Command"
        My-Exit
    }

    $url = "https://{0}:{1}/{2}" -f $casHostname, $casPort, $instanceID
    
    if ($casHttpThumbprint -ceq ''){
        $cmdout = &$MailStoreCmd --h=$url --u=$instanceUsername --p=$instancePassword --nologo -c $command
    } else {
        $cmdout = &$MailStoreCmd --h=$url --u=$instanceUsername --p=$instancePassword --pkv3=$casHttpThumbprint --nologo -c $command
    }

    if($?) {
       #MailStoreCmd.exe succeeded
       $jsonresult = $cmdout -replace '(.*)\.\.\.', ""| Where-Object { $_ -ne "" }  #Strip connecting... disconnecting... installing...
       try {
           $result = ConvertFrom-Json (-join $jsonresult) -ErrorAction Stop;
       }
       catch {
           $result = $jsonresult
           Write-Host $jsonresult
       }
    }
    else {
        Write-Host "MailStoreCmd.exe failed"
        Write-Host ($cmdout -replace '(.*)\.\.\.', ""| Where-Object { $_ -ne "" })
        My-Exit
    }
    #Write-Host $result
    return $result
}

Write-Host "Creating API client"
$msapiclient = New-MSApiClient -Username $speUsername -Password $spePassword -ManagementServer $mgmtHostname -Port $mgmtPort -IgnoreInvalidSSLCerts:$mgmtAcceptAllCertificates

Write-Host "Checking for write-protected archive stores"
$archive_stores_result = Invoke-MSApiCall $msapiclient "GetStores"  @{"instanceID" = $instanceID}

if (-not (Has-Succeeded($archive_stores_result))) {
    My-Exit
}

if ($archive_stores_result.result.requestedState -match "writeProtected") {
    foreach ($archive_store in $archive_stores_result.result){
        if ($archive_store.requestedState -ceq "writeProtected") {
            Write-Host ("Archive store with ID: " + $archive_store.id + " Name: '" + $archive_store.name + "' is write protected")
        }
    }
    Write-Host ("At least one write-protected archive store has been found. Script aborting.")
    My-Exit
}

Write-Host ("Reading folders file " + $filename)
foreach ($line in (Get-Content $filename))
{
    # Skip lines starting with # and empty lines
    if ($line.StartsWith("#") -or -not $line) {
        Write-Host ("Skipping line '" + $line + "'")
        continue
    }
    
    $foldername_parts = $line.Split("=")

    # skip folders which contain at least one equals character
    if ($foldername_parts.count -gt 2) {
		Write-Host ("Source foldername or target foldername contains at least one equals character (=). Line: {0}. Skipping" -f $line)
		continue
	}

    $source_folder = $foldername_parts[0]
    $target_folder = $foldername_parts[1]

    # skip folders without source
    if (-not $source_folder){
        Write-Host ("Source foldername of '" + $target_folder + "' is empty. Skipping")
        continue
    }

    # skip folders without target
    if (-not $target_folder){
        Write-Host ("Target foldername of '" + $source_folder + "' is empty. Skipping folder")
        continue
    }    
    
    # skip folders that do not change
    if ($source_folder -ceq $target_folder){
        Write-Host ("Old folder name and new folder name must not match, but they do: " + $source_folder + " = " + $target_folder)
        continue
    }

    Write-Host ("Moving folder " + $source_folder + " to " + $target_folder)
    $rename_folder_result = CmdClient("MoveFolder", "--fromFolder=$source_folder", "--toFolder=$target_folder")
    
    # Write-Host $rename_folder_result.statusCode
}

My-Exit
